# -*- coding: utf-8; -*-
#
# Copyright (c) 2016 Álan Crístoffer
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

'''
Decorators used by the moirai server to control logging and exception handling.
'''

import errno
import inspect
import os
import socket
import time

import appdirs

import moirai

LOG_DIR = appdirs.user_log_dir(
    appname=moirai.__name__,
    appauthor=moirai.__author__,
    version=moirai.__version__)
os.makedirs(LOG_DIR, exist_ok=True)
LOG_FILE = open(os.path.join(LOG_DIR, 'moirai.log'), 'at')


def log_file_path():
    """
    Returns the path of the log file.
    """
    return os.path.join(LOG_DIR, 'moirai.log')


def decorate_all_methods(decorator):
    """
    Decorates every member of a class.
    """

    def decorate(cls):
        """
        Decorates every member of a class.
        """
        for name, func in inspect.getmembers(cls, inspect.isfunction):
            setattr(cls, name, decorator(func))
        return cls

    return decorate


def ignore_eagain(func):
    """
    errno.EAGAIN is generated in some platforms when the socket is
    non-blocking. This decorator ignores this exception, as it's not an error.
    """

    def decorate(ret, *kargs):
        """
        Decorator.
        """
        try:
            return func(*kargs)
        except socket.error as error:
            if error.errno == errno.EAGAIN:
                return ret
            raise error

    return decorate


def log_msg(msg):
    """
    Logs `msg` to log file.
    """
    msg = '%s: %s\n' % (time.ctime(), msg)
    LOG_FILE.write(msg)
    LOG_FILE.flush()


def log(func):
    """
    Logs every call to a class method.
    """

    def decorate(*kargs, **kwargs):
        """
        Logs every call to a class method.
        """
        func_name = []
        if func.__module__ != '__main__':
            func_name.append(func.__module__)
        func_name.append(func.__qualname__)
        func_name = '.'.join(func_name)

        args = []
        if kargs:
            args = [str(arg) for arg in kargs]
        if kwargs:
            args += ['%s=%s' % i for i in kwargs.items()]
        args = ', '.join(args)

        log_str = '%s(%s)' % (func_name, args)
        log_msg(log_str)
        return func(*kargs, **kwargs)

    return decorate


def dont_raise(func):
    """
    Decorate every member of a class to stop raising exceptions.
    """

    def decorate(*kargs, **kwargs):
        """
        Decorate every member of a class to stop raising exceptions.
        """
        try:
            return func(*kargs, **kwargs)
        except Exception as error:
            log_msg('An Exception occurred: %s' % error)
            print('An Exception occurred: %s' % error)

    return decorate



#!/usr/bin/env python3
# -*- coding: utf-8; -*-
#
# Copyright (c) 2016 Álan Crístoffer
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

'''
Moirai server installation functions.
'''

import glob
import os
import platform
import re
import shutil
import sys
import zipfile

from urllib.request import urlopen
from pathlib import Path

opt = os.path.join(os.path.abspath(os.sep), 'opt')


def pi_version():
    """Detect the version of the Raspberry Pi.  Returns either 1, 2 or
    None depending on if it's a Raspberry Pi 1 (model A, B, A+, B+),
    Raspberry Pi 2 (model B+), or not a Raspberry Pi.
    https://github.com/adafruit/Adafruit_Python_GPIO/blob/master/Adafruit_GPIO/Platform.py
    """
    if not os.path.isfile('/proc/cpuinfo'):
        return None
    # Check /proc/cpuinfo for the Hardware field value.
    # 2708 is pi 1
    # 2709 is pi 2
    # Anything else is not a pi.
    with open('/proc/cpuinfo', 'r') as infile:
        cpuinfo = infile.read()
    # Match a line like 'Hardware   : BCM2709'
    match = re.search(
        r'^Hardware\s+:\s+(\w+)$', cpuinfo, flags=re.MULTILINE | re.IGNORECASE)
    if not match:
        # Couldn't find the hardware, assume it isn't a pi.
        return None
    if match.group(1) == 'BCM2708':
        # Pi 1
        return 1
    if match.group(1) == 'BCM2709':
        # Pi 2
        return 2
    if match.group(1) == 'BCM2835':
        # Pi 3
        return 3
    # Something else, not a pi.
    return None


class CD:
    """Context manager for changing the current working directory"""

    def __init__(self, new_path):
        self.new_path = os.path.expanduser(new_path)
        self.saved_path = None

    def __enter__(self):
        self.saved_path = os.getcwd()
        os.chdir(self.new_path)

    def __exit__(self, etype, value, traceback):
        os.chdir(self.saved_path)


def download_mysql_connector(use_sudo):
    '''
    Downloads MySQL Connector
    '''
    sudo = 'sudo' if use_sudo else ''
    print('Downloading MySQL Connector/Python')
    try:
        url = (
            'https://dev.mysql.com/get/Downloads/Connector-Python'
            '/mysql-connector-python-2.1.7.tar.gz'
        )
        contents = urlopen(url).read()
        with open('mysql.txz', 'wb') as file:
            file.write(contents)
        os.system('tar xf mysql.txz')
        with CD('mysql-connector-python-2.1.7'):
            os.system('%s %s setup.py install' % (sudo, sys.executable))
        return True
    except:  # noqa: E722 pylint: disable=E722
        return False


def download_snap7_linux(use_sudo):
    '''
    Downloads Snap7 for Linux
    '''
    sudo = 'sudo' if use_sudo else ''
    arch = {
        'x64': 'x86_64',
        'ia32': 'i386',
        'x86': 'i386',
        'armv7': 'arm_v7',
        'armv6': 'arm_v6'
    }.get(platform.machine(), platform.machine())
    print('Downloading snap7...')
    try:
        if not os.path.exists('snap7.zip'):
            snap7url = (
                'https://sourceforge.net/projects/snap7/'
                'files/1.4.2/snap7-full-1.4.2.7z/download'
            )
            contents = urlopen(snap7url).read()
            with open('snap7.zip', 'wb') as file:
                file.write(contents)
        os.system('7z x -y snap7.zip')
        with CD('snap7-full-1.4.2/build/unix'):
            os.system(f'make -f {arch}_linux.mk')
        lib = f'snap7-full-1.4.2/build/bin/{arch}-linux/libsnap7.so'
        os.system(f'{sudo} cp {lib} {os.path.join(opt, "libsnap7.so")}')
        os.system(f'{sudo} chmod 777 {os.path.join(opt, "libsnap7.so")}')
        os.system(f'echo "{opt}" | {sudo} tee -a /etc/ld.so.conf.d/moirai')
        os.system(f'{sudo} ldconfig')
        os.remove('snap7.zip')
        shutil.rmtree('snap7-full-1.4.2')
        return True
    except:  # noqa: E722 pylint: disable=E722
        return False


def download_snap7_win():
    '''
    Downloads Snap7 for Windows
    '''
    print('Downloading snap7...')
    try:
        snap7url = (
            'https://sourceforge.net/projects/snap7/'
            'files/latest/download?source=files'
        )
        contents = urlopen(snap7url).read()
        with open('snap7.zip', 'wb') as file:
            file.write(contents)
        with zipfile.ZipFile('snap7.zip', 'r') as zip_ref:
            zip_ref.extractall('snap7')
        dll = list(glob.iglob('snap7/**/*.dll'))[0]
        shutil.copy(dll, os.path.join(opt, 'snap7.dll'))
        os.remove('snap7.zip')
        shutil.rmtree('snap7')
        return True
    except:  # noqa: E722 pylint: disable=E722
        return False


def install_macos():
    '''
    Installs on macOS
    '''
    print('Platform: macOS')
    brew = os.popen('which brew').read()
    if len(brew) > 0:
        print('Using Homebrew to install dependencies.')
        print('Adding tap mongodb/brew')
        os.system('brew tap mongodb/brew')
        print('Will now execute: [brew install snap7 mongodb-community]')
        result = os.system('brew install snap7 mongodb-community')
        if result == 0:
            print('Installation finished.')
        else:
            print('Something went wrong. Try installing manually:')
            print('brew install snap7 mongodb-community')
    else:
        print('Homebrew not installed, cannot continue alone.')
        print('You need to install MongoDB and snap7.')
        print('https://www.mongodb.com')
        print('https://sourceforge.net/projects/snap7/files')


def install_raspberry(sudo, use_sudo):
    '''
    Installs on Raspberry Pi
    '''
    print('Platform: Raspberry Pi')
    cmd = sudo + (
        'DEBIAN_FRONTEND=noninteractive apt-get install -y '
        'mysql-server build-essential p7zip-full'
    )
    print('Using APT to install dependencies.')
    print(f'Will now execute: [{cmd}]')
    result = os.system(cmd)
    config_dir = str(Path.home() / '.config')
    xdg_config = os.environ.get('XDG_CONFIG_HOME', config_dir)
    config_file = str(Path(xdg_config) / 'moirai' / 'config.json')
    with open(config_file, 'a+') as file:
        file.seek(0)
        contents = file.read()
        if not contents:
            file.write('{"database":{"adapter":"mysql"}}')
    if result == 0:
        mysql = download_mysql_connector(use_sudo)
        snap7 = download_snap7_linux(use_sudo)
        if mysql and snap7:
            print('Installation finished.')
        else:
            print('Something went wrong. Try installing manually:')
            print('https://sourceforge.net/projects/snap7/files')
    else:
        print('Something went wrong. Try installing manually:')
        print(cmd)
        print('After that, you need to install snap7.')
        print('https://sourceforge.net/projects/snap7/files')


def install_open_suse(sudo, use_sudo):
    '''
    Installs on openSUSE
    '''
    print('Platform: OpenSuse')
    cmd1 = sudo + 'zypper -n in -t pattern devel_basis'
    cmd2 = sudo + 'zypper -n in mongodb p7zip'
    print('Using Zypper to install dependencies.')
    print(f'Will now execute: [{cmd1}]')
    result1 = os.system(cmd1)
    print(f'Will now execute: [{cmd2}]')
    result2 = os.system(cmd2)
    if result1 == 0 and result2 == 0:
        if download_snap7_linux(use_sudo):
            print('Installation finished.')
        else:
            print('Something went wrong. Try installing manually:')
            print('https://sourceforge.net/projects/snap7/files')
    else:
        print('Something went wrong. Try installing manually:')
        print(cmd1)
        print(cmd2)
        print('After that, you need to install snap7.')
        print('https://sourceforge.net/projects/snap7/files')


def install_debian(sudo, use_sudo):
    '''
    Intalls on Debian
    '''
    print('Platform: Debian')
    cmd = sudo + 'apt-get install -y mongodb-server build-essential p7zip-full'
    print('Using APT to install dependencies.')
    print(f'Will now execute: [{cmd}]')
    result = os.system(cmd)
    if result == 0:
        if download_snap7_linux(use_sudo):
            print('Installation finished.')
        else:
            print('Something went wrong. Try installing manually:')
            print('https://sourceforge.net/projects/snap7/files')
    else:
        print('Something went wrong. Try installing manually:')
        print(cmd)
        print('After that, you need to install snap7.')
        print('https://sourceforge.net/projects/snap7/files')


def install_fedora_dnf(sudo, use_sudo):
    '''
    Intalls on Fedora using DNF
    '''
    print('Platform: Fedora')
    cmd = sudo + 'dnf install -y @development-tools p7zip mongodb-server'
    print('Using DNF to install dependencies.')
    print(f'Will now execute: [{cmd}]')
    result = os.system(cmd)
    if result == 0:
        if download_snap7_linux(use_sudo):
            print('Installation finished.')
        else:
            print('Something went wrong. Try installing manually:')
            print('https://sourceforge.net/projects/snap7/files')
    else:
        print('Something went wrong. Try installing manually:')
        print(cmd)
        print('After that, you need to install snap7.')
        print('https://sourceforge.net/projects/snap7/files')


def install_fedora_yum(sudo, use_sudo):
    '''
    Installs on Fedora using YUM
    '''
    print('Platform: Fedora')
    cmd1 = sudo + (
        'yum groupinstall -y '
        '"Development Tools" "Development Libraries"'
    )
    cmd2 = sudo + 'yum install -y p7zip mongodb-server'
    print('Using YUM to install dependencies.')
    print(f'Will now execute: [{cmd1}]')
    result1 = os.system(cmd1)
    print(f'Will now execute: [{cmd2}]')
    result2 = os.system(cmd2)
    if result1 == 0 and result2 == 0:
        if download_snap7_linux(use_sudo):
            print('Installation finished.')
        else:
            print('Something went wrong. Try installing manually:')
            print('https://sourceforge.net/projects/snap7/files')
    else:
        print('Something went wrong. Try installing manually:')
        print(cmd1)
        print(cmd2)
        print('After that, you need to install snap7.')
        print('https://sourceforge.net/projects/snap7/files')


def install_linux(use_sudo):
    '''
    Installs on Linux
    '''
    sudo = 'sudo ' if use_sudo else ''
    if pi_version() is not None:
        install_raspberry(sudo, use_sudo)
    elif len(os.popen('which zypper').read()) > 0:
        install_open_suse(sudo, use_sudo)
    elif len(os.popen('which apt-get').read()) > 0:
        install_debian(sudo, use_sudo)
    elif len(os.popen('which dnf').read()) > 0:
        install_fedora_dnf(sudo, use_sudo)
    elif len(os.popen('which yum').read()) > 0:
        install_fedora_yum(sudo, use_sudo)
    else:
        print('You need to install MongoDB and snap7.')
        print('https://www.mongodb.com')
        print('https://sourceforge.net/projects/snap7/files')


def install_windows():
    '''
    Intalls on Windows
    '''
    print('Trying to install MongoDB and snap7...')
    if download_snap7_win():
        print('Downloading MongoDB...')
        mongo = (
            'https://fastdl.mongodb.org/win32/'
            'mongodb-win32-x86_64-2008plus-ssl-4.0.1-signed.msi'
        )
        contents = urlopen(mongo).read()
        with open('mongo.msi', 'wb') as file:
            file.write(contents)
        result = os.system(
            r'msiexec.exe /q /i mongo.msi INSTALLLOCATION='
            r'"C:\Program Files\MongoDB\Server\4.0.1\" ADDLOCAL="all"'
        )
        if result:
            print('Installation finished.')
        else:
            print('You need to install MongoDB.')
            print('https://www.mongodb.com')
    else:
        print('You need to install MongoDB and snap7.')
        print('https://www.mongodb.com')
        print('https://sourceforge.net/projects/snap7/files')


def install(use_sudo=False):
    '''
    Installs moirai dependencies
    '''
    try:
        osname = platform.system()
        if osname == 'Windows':
            print('You need to install MongoDB.')
            print('https://www.mongodb.com')
            print('')
        elif osname == 'Darwin':
            install_macos()
        elif osname == 'Linux':
            install_linux(use_sudo)
        elif osname == 'Windows':
            install_windows()
        else:
            print('You need to install MongoDB and snap7.')
            print('https://www.mongodb.com')
            print('https://sourceforge.net/projects/snap7/files')
    except:  # noqa: E722 pylint: disable=E722
        print('Something went wrong. Try installing manually.')
        print('You need to install MongoDB or MySQL and snap7.')
        print('https://www.mongodb.com')
        print('https://sourceforge.net/projects/snap7/files')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['moirai', 'moirai.database', 'moirai.hardware', 'moirai.webapi']

package_data = \
{'': ['*']}

install_requires = \
['Flask>=2.0.1,<3.0.0',
 'ahio>=1.0.30,<2.0.0',
 'appdirs>=1.4.4,<2.0.0',
 'cheroot>=8.5.2,<9.0.0',
 'numpy>=1.21.0,<2.0.0',
 'pymongo>=3.11.4,<4.0.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'scipy>=1.9.0,<2.0.0']

entry_points = \
{'console_scripts': ['moirai = moirai.moirai:start']}

setup_kwargs = {
    'name': 'moirai',
    'version': '1.3.23',
    'description': 'Digital Control Manager Backend',
    'long_description': '# moirai\n\nMoirai is the backend for the platform. It is developed as part of my scientific\ninitiation project, named Plataformas de baixo custo para controle de processos\n(low-cost platform for process control), developed at CEFET-MG (Brazil) under\nthe supervision of Prof. Dr. Valter Leite. The project was developed through a\nFAPEMIG scholaship.\n\nIt\'s meant to be installed in a computer near the plant, so it can be\nremote-controlled by [Lachesis](https://github.com/acristoffers/Lachesis). It\nexposes a RESTful API that let\'s you configure the hardware, save tests and\ncontrollers, execute them and fetch the results in real-time or after the\nexecution finishes.\n\nControllers for this platform are written in Python 3 and can use any librarie\navailable in the computer where _moirai_ is running. It already depends on NumPy\nand SciPy, as they are commonly used. This platform manages the hardware\ninterfacing through the [ahio](https://github.com/acristoffers/ahio) libray, so\nextending it\'s hardware capabilities is a question of extending AHIO, which\nshould be simple. Execution time, sampling time, input querying and output\nupdating is managed by the application and let\'s you focus on your\ncontroller/model.\n\n## Installation\n\nUse pip to install. This is a Python 3 application and won\'t run in Python 2.\nUse `pip install moirai` or `pip3 install moirai` to install it. It also has\nother dependencies not installable through pip, which can be installed by\nrunning `moirai --install --sudo`. It will install MongoDB (or MySQL on\nRaspberry Pi) and the Snap7 library. It\'s designed to work on Windows, macOS\n(with Homebrew) and Linux (apt-get, dnf, yum and zypper).\n\nIf using the Snap7 driver on Windows, you may need to compile the driver\nyourself and copy it to /Windows/System32.\n\nOn macOS, using Homebrew, you can install with:\n\n```bash\nbrew tap acristoffers/repo\nbrew install moirai\n```\n\n## License\n\nCopyright (c) 2016 Álan Crístoffer\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the "Software"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.\n',
    'author': 'Álan Crístoffer',
    'author_email': 'acristoffers@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/acristoffers/moirai',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.12',
}


setup(**setup_kwargs)
